@@ -1,31 +1,23 @@
 /*
- * "$Id: test1284.c 5591 2006-05-26 19:51:59Z mike $"
+ * "$Id: test1284.c 7465 2008-04-18 16:20:11Z mike $"
  *
  *   IEEE-1284 support functions test program for the Common UNIX Printing
  *   System (CUPS).
  *
+ *   Copyright 2007 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
  *   "LICENSE" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  *
  * Contents:
  *
+ *   main() - Test the device-ID functions.
  */
 
 /*
@@ -42,7 +34,6 @@
 #  include <fcntl.h>
 #endif /* WIN32 */
 
-#define DEBUG
 #include "ieee1284.c"
 
 
@@ -92,5 +83,5 @@ main(int  argc,				/* I - Number of command-line args */
 
 
 /*
- * End of "$Id: test1284.c 5591 2006-05-26 19:51:59Z mike $".
+ * End of "$Id: test1284.c 7465 2008-04-18 16:20:11Z mike $".
  */
