@@ -1,5 +1,5 @@
 /*
-* "$Id: usb-darwin.c 8855 2009-10-26 17:20:01Z mike $"
+* "$Id: usb-darwin.c 8896 2009-11-20 01:27:57Z mike $"
 *
 * Copyright 2005-2009 Apple Inc. All rights reserved.
 *
@@ -373,12 +373,12 @@ print_device(const char *uri,		/* I - Device URI */
 
   if (!g.make || !g.model)
   {
-    _cupsLangPuts(stderr, _("ERROR: Fatal USB error!\n"));
+    _cupsLangPuts(stderr, _("ERROR: Fatal USB error\n"));
 
     if (!g.make)
-      fputs("DEBUG: USB make string is NULL!\n", stderr);
+      fputs("DEBUG: USB make string is NULL\n", stderr);
     if (!g.model)
-      fputs("DEBUG: USB model string is NULL!\n", stderr);
+      fputs("DEBUG: USB model string is NULL\n", stderr);
 
     return (CUPS_BACKEND_STOP);
   }
@@ -431,7 +431,7 @@ print_device(const char *uri,		/* I - Device URI */
         strlcpy(print_buffer, "USB class driver", sizeof(print_buffer));
 
       fputs("STATE: +apple-missing-usbclassdriver-error\n", stderr);
-      _cupsLangPuts(stderr, _("ERROR: Fatal USB error!\n"));
+      _cupsLangPuts(stderr, _("ERROR: Fatal USB error\n"));
       fprintf(stderr, "DEBUG: Could not load %s\n", print_buffer);
 
       if (driverBundlePath)
@@ -496,8 +496,9 @@ print_device(const char *uri,		/* I - Device URI */
 
     if (pthread_create(&sidechannel_thread_id, NULL, sidechannel_thread, NULL))
     {
-      _cupsLangPuts(stderr, _("ERROR: Fatal USB error!\n"));
-      fputs("DEBUG: Couldn't create side-channel thread!\n", stderr);
+      _cupsLangPuts(stderr, _("ERROR: Fatal USB error\n"));
+      fputs("DEBUG: Couldn't create side-channel thread\n", stderr);
+      registry_close();
       return (CUPS_BACKEND_STOP);
     }
   }
@@ -514,8 +515,9 @@ print_device(const char *uri,		/* I - Device URI */
 
   if (pthread_create(&read_thread_id, NULL, read_thread, NULL))
   {
-    _cupsLangPuts(stderr, _("ERROR: Fatal USB error!\n"));
-    fputs("DEBUG: Couldn't create read thread!\n", stderr);
+    _cupsLangPuts(stderr, _("ERROR: Fatal USB error\n"));
+    fputs("DEBUG: Couldn't create read thread\n", stderr);
+    registry_close();
     return (CUPS_BACKEND_STOP);
   }
 
@@ -593,14 +595,16 @@ print_device(const char *uri,		/* I - Device URI */
 	if (errno == EINTR && total_bytes == 0)
 	{
 	  fputs("DEBUG: Received an interrupt before any bytes were "
-	        "written, aborting!\n", stderr);
+	        "written, aborting\n", stderr);
+          registry_close();
           return (CUPS_BACKEND_OK);
 	}
 	else if (errno != EAGAIN && errno != EINTR)
 	{
-	  _cupsLangPuts(stderr, _("ERROR: Unable to read print data!\n"));
+	  _cupsLangPuts(stderr, _("ERROR: Unable to read print data\n"));
 	  perror("DEBUG: select");
-	  return (CUPS_BACKEND_FAILED);
+	  registry_close();
+          return (CUPS_BACKEND_FAILED);
 	}
       }
 
@@ -640,8 +644,9 @@ print_device(const char *uri,		/* I - Device URI */
 
 	  if (errno != EAGAIN && errno != EINTR)
 	  {
-	    _cupsLangPuts(stderr, _("ERROR: Unable to read print data!\n"));
+	    _cupsLangPuts(stderr, _("ERROR: Unable to read print data\n"));
 	    perror("DEBUG: read");
+	    registry_close();
 	    return (CUPS_BACKEND_FAILED);
 	  }
 
@@ -674,7 +679,7 @@ print_device(const char *uri,		/* I - Device URI */
 
 	if (iostatus == kIOUSBTransactionTimeout)
 	{
-	  fputs("DEBUG: Got USB transaction timeout during write!\n", stderr);
+	  fputs("DEBUG: Got USB transaction timeout during write\n", stderr);
 	  iostatus = 0;
 	}
 
@@ -684,7 +689,7 @@ print_device(const char *uri,		/* I - Device URI */
 
 	else if (iostatus == kIOUSBPipeStalled)
 	{
-	  fputs("DEBUG: Got USB pipe stalled during write!\n", stderr);
+	  fputs("DEBUG: Got USB pipe stalled during write\n", stderr);
 
 	  bytes    = g.print_bytes;
 	  iostatus = (*g.classdriver)->WritePipe(g.classdriver, (UInt8*)print_ptr, &bytes, 0);
@@ -697,7 +702,7 @@ print_device(const char *uri,		/* I - Device URI */
 
 	else if (iostatus == kIOReturnAborted)
 	{
-	  fputs("DEBUG: Got USB return aborted during write!\n", stderr);
+	  fputs("DEBUG: Got USB return aborted during write\n", stderr);
 
 	  IOReturn err = (*g.classdriver)->Abort(g.classdriver);
 	  fprintf(stderr, "DEBUG: USB class driver Abort returned %x\n", err);
@@ -716,7 +721,7 @@ print_device(const char *uri,		/* I - Device URI */
 	  * Write error - bail if we don't see an error we can retry...
 	  */
 
-	  _cupsLangPuts(stderr, _("ERROR: Unable to send print data!\n"));
+	  _cupsLangPuts(stderr, _("ERROR: Unable to send print data\n"));
 	  fprintf(stderr, "DEBUG: USB class driver WritePipe returned %x\n",
 	          iostatus);
 
@@ -897,11 +902,11 @@ static void *read_thread(void *reference)
 #endif
     }
     else if (readstatus == kIOUSBTransactionTimeout)
-      fputs("DEBUG: Got USB transaction timeout during read!\n", stderr);
+      fputs("DEBUG: Got USB transaction timeout during read\n", stderr);
     else if (readstatus == kIOUSBPipeStalled)
-      fputs("DEBUG: Got USB pipe stalled during read!\n", stderr);
+      fputs("DEBUG: Got USB pipe stalled during read\n", stderr);
     else if (readstatus == kIOReturnAborted)
-      fputs("DEBUG: Got USB return aborted during read!\n", stderr);
+      fputs("DEBUG: Got USB return aborted during read\n", stderr);
 
    /*
     * Make sure this loop executes no more than once every 250 miliseconds...
@@ -943,7 +948,12 @@ sidechannel_thread(void *reference)
     datalen = sizeof(data);
 
     if (cupsSideChannelRead(&command, &status, data, &datalen, 1.0))
-      break;
+    {
+      if (status == CUPS_SC_STATUS_TIMEOUT)
+	continue;
+      else
+	break;
+    }
 
     switch (command)
     {
@@ -1901,6 +1911,7 @@ static void run_legacy_backend(int argc,
     * Setup a SIGTERM handler then block it before forking...
     */
 
+    int			err;		/* posix_spawn result */
     struct sigaction	action;		/* POSIX signal action */
     sigset_t		newmask,	/* New signal mask */
 			oldmask;	/* Old signal mask */
@@ -1949,7 +1960,7 @@ static void run_legacy_backend(int argc,
 #  else
 	perror("DEBUG: Unable to set binary preference to ppc");
 #  endif /* __x86_64__ */
-	_cupsLangPrintf(stderr, _("Unable to use legacy USB class driver!\n"));
+	_cupsLangPrintf(stderr, _("Unable to use legacy USB class driver\n"));
 	exit(CUPS_BACKEND_STOP);
       }
     }
@@ -1967,11 +1978,12 @@ static void run_legacy_backend(int argc,
 
     my_argv[i] = NULL;
 
-    if (posix_spawn(&child_pid, usbpath, NULL, &attrs, my_argv, environ))
+    if ((err = posix_spawn(&child_pid, usbpath, NULL, &attrs, my_argv,
+                           environ)) != 0)
     {
       fprintf(stderr, "DEBUG: Unable to exec %s: %s\n", usbpath,
-              strerror(errno));
-      _cupsLangPrintf(stderr, _("Unable to use legacy USB class driver!\n"));
+              strerror(err));
+      _cupsLangPrintf(stderr, _("Unable to use legacy USB class driver\n"));
       exit(CUPS_BACKEND_STOP);
     }
 
@@ -1997,14 +2009,14 @@ static void run_legacy_backend(int argc,
     if (WIFSIGNALED(childstatus))
     {
       exitstatus = CUPS_BACKEND_STOP;
-      fprintf(stderr, "DEBUG: usb(legacy) backend %d crashed on signal %d!\n",
+      fprintf(stderr, "DEBUG: usb(legacy) backend %d crashed on signal %d\n",
               child_pid, WTERMSIG(childstatus));
     }
     else
     {
       if ((exitstatus = WEXITSTATUS(childstatus)) != 0)
 	fprintf(stderr,
-	        "DEBUG: usb(legacy) backend %d stopped with status %d!\n",
+	        "DEBUG: usb(legacy) backend %d stopped with status %d\n",
 		child_pid, exitstatus);
       else
 	fprintf(stderr, "DEBUG: usb(legacy) backend %d exited with no errors\n",
@@ -2046,7 +2058,7 @@ sigterm_handler(int sig)		/* I - Signal */
       exit(0);
     else
     {
-      fprintf(stderr, "DEBUG: Child crashed on signal %d!\n", status);
+      fprintf(stderr, "DEBUG: Child crashed on signal %d\n", status);
       exit(CUPS_BACKEND_STOP);
     }
   }
@@ -2219,5 +2231,5 @@ static void get_device_id(cups_sc_status_t *status,
 
 
 /*
- * End of "$Id: usb-darwin.c 8855 2009-10-26 17:20:01Z mike $".
+ * End of "$Id: usb-darwin.c 8896 2009-11-20 01:27:57Z mike $".
  */
