@@ -1,5 +1,5 @@
 /*
- * "$Id: jobs.c 7238 2008-01-22 01:41:31Z mike $"
+ * "$Id: jobs.c 8859 2009-11-09 23:01:17Z mike $"
  *
  *   Job status CGI for the Common UNIX Printing System (CUPS).
  *
@@ -57,6 +57,7 @@ main(int  argc,				/* I - Number of command-line arguments */
   */
 
   cgiSetVariable("SECTION", "jobs");
+  cgiSetVariable("REFRESH_PAGE", "");
 
  /*
   * Connect to the HTTP server...
@@ -77,7 +78,7 @@ main(int  argc,				/* I - Number of command-line arguments */
   * Do the operation...
   */
 
-  if ((op = cgiGetVariable("OP")) != NULL && job_id > 0)
+  if ((op = cgiGetVariable("OP")) != NULL && job_id > 0 && cgiIsPOST())
   {
    /*
     * Do the operation...
@@ -209,5 +210,5 @@ do_job_op(http_t      *http,		/* I - HTTP connection */
 
 
 /*
- * End of "$Id: jobs.c 7238 2008-01-22 01:41:31Z mike $".
+ * End of "$Id: jobs.c 8859 2009-11-09 23:01:17Z mike $".
  */
