@@ -1,5 +1,5 @@
 /*
- * "$Id: adminutil.h 6649 2007-07-11 21:46:42Z mike $"
+ * "$Id: adminutil.h 7026 2007-10-19 00:57:45Z mike $"
  *
  *   Administration utility API definitions for the Common UNIX Printing
  *   System (CUPS).
@@ -58,16 +58,16 @@ extern int	cupsAdminExportSamba(const char *dest, const char *ppd,
 		                     const char *samba_server,
 			             const char *samba_user,
 				     const char *samba_password,
-				     FILE *logfile);
+				     FILE *logfile) _CUPS_API_1_2;
 extern char	*cupsAdminCreateWindowsPPD(http_t *http, const char *dest,
-		                           char *buffer, int bufsize);
+		                           char *buffer, int bufsize) _CUPS_API_1_2;
 
 extern int	cupsAdminGetServerSettings(http_t *http,
 			                   int *num_settings,
-		                           cups_option_t **settings);
+		                           cups_option_t **settings) _CUPS_API_1_3;
 extern int	cupsAdminSetServerSettings(http_t *http,
 		                           int num_settings,
-		                           cups_option_t *settings);
+		                           cups_option_t *settings) _CUPS_API_1_3;
 
 
 #  ifdef __cplusplus
@@ -77,5 +77,5 @@ extern int	cupsAdminSetServerSettings(http_t *http,
 #endif /* !_CUPS_ADMINUTIL_H_ */
 
 /*
- * End of "$Id: adminutil.h 6649 2007-07-11 21:46:42Z mike $".
+ * End of "$Id: adminutil.h 7026 2007-10-19 00:57:45Z mike $".
  */
