@@ -1,9 +1,9 @@
 /*
- * "$Id: localize.c 8374 2009-02-20 16:42:13Z mike $"
+ * "$Id: localize.c 8627 2009-05-13 21:39:17Z mike $"
  *
- *   PPD custom option routines for the Common UNIX Printing System (CUPS).
+ *   PPD localization routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 2007 by Apple Inc.
+ *   Copyright 2007-2009 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -25,11 +25,18 @@
  *
  * Contents:
  *
- *   ppdLocalize()          - Localize the PPD file to the current locale.
- *   ppdLocalizeIPPReason() - Get the localized version of a cupsIPPReason
- *                            attribute.
- *   ppd_ll_CC()            - Get the current locale names.
- *   ppd_localized_attr()   - Find a localized attribute.
+ *   ppdLocalize()           - Localize the PPD file to the current locale.
+ *   ppdLocalizeAttr()       - Localize an attribute.
+ *   ppdLocalizeIPPReason()  - Get the localized version of a cupsIPPReason
+ *                             attribute.
+ *   ppdLocalizeMarkerName() - Get the localized version of a marker-names
+ *                             attribute value.
+ *   _ppdFreeLanguages()     - Free an array of languages from _ppdGetLanguages.
+ *   _ppdGetLanguages()      - Get an array of languages from a PPD file.
+ *   _ppdHashName()          - Generate a hash value for a device or profile
+ *                             name.
+ *   _ppdLocalizedAttr()     - Find a localized attribute.
+ *   ppd_ll_CC()             - Get the current locale names.
  */
 
 /*
@@ -37,6 +44,7 @@
  */
 
 #include "globals.h"
+#include "ppd-private.h"
 #include "debug.h"
 
 
@@ -44,12 +52,7 @@
  * Local functions...
  */
 
-static void		ppd_ll_CC(char *ll_CC, int ll_CC_size,
-			          char *ll, int ll_size);
-static ppd_attr_t	*ppd_localized_attr(ppd_file_t *ppd,
-			                    const char *keyword,
-			                    const char *spec, const char *ll_CC,
-				            const char *ll);
+static cups_lang_t	*ppd_ll_CC(char *ll_CC, int ll_CC_size);
 
 
 /*
@@ -59,7 +62,7 @@ static ppd_attr_t	*ppd_localized_attr(ppd_file_t *ppd,
  * descriptions, printer presets, and custom option parameters.  Each
  * localized string uses the UTF-8 character encoding.
  *
- * @since CUPS 1.2@
+ * @since CUPS 1.2/Mac OS X 10.5@
  */
 
 int					/* O - 0 on success, -1 on error */
@@ -74,15 +77,14 @@ ppdLocalize(ppd_file_t *ppd)		/* I - PPD file */
   ppd_attr_t	*attr,			/* Current attribute */
 		*locattr;		/* Localized attribute */
   char		ckeyword[PPD_MAX_NAME],	/* Custom keyword */
-		ll_CC[6],		/* Language + country locale */
-		ll[3];			/* Language locale */
+		ll_CC[6];		/* Language + country locale */
 
 
  /*
   * Range check input...
   */
 
-  DEBUG_printf(("ppdLocalize(ppd=%p)\n", ppd));
+  DEBUG_printf(("ppdLocalize(ppd=%p)", ppd));
 
   if (!ppd)
     return (-1);
@@ -91,7 +93,7 @@ ppdLocalize(ppd_file_t *ppd)		/* I - PPD file */
   * Get the default language...
   */
 
-  ppd_ll_CC(ll_CC, sizeof(ll_CC), ll, sizeof(ll));
+  ppd_ll_CC(ll_CC, sizeof(ll_CC));
 
  /*
   * Now lookup all of the groups, options, choices, etc.
@@ -99,14 +101,14 @@ ppdLocalize(ppd_file_t *ppd)		/* I - PPD file */
 
   for (i = ppd->num_groups, group = ppd->groups; i > 0; i --, group ++)
   {
-    if ((locattr = ppd_localized_attr(ppd, "Translation", group->name,
-                                      ll_CC, ll)) != NULL)
+    if ((locattr = _ppdLocalizedAttr(ppd, "Translation", group->name,
+                                     ll_CC)) != NULL)
       strlcpy(group->text, locattr->text, sizeof(group->text));
 
     for (j = group->num_options, option = group->options; j > 0; j --, option ++)
     {
-      if ((locattr = ppd_localized_attr(ppd, "Translation", option->keyword,
-                                        ll_CC, ll)) != NULL)
+      if ((locattr = _ppdLocalizedAttr(ppd, "Translation", option->keyword,
+                                       ll_CC)) != NULL)
 	strlcpy(option->text, locattr->text, sizeof(option->text));
 
       for (k = option->num_choices, choice = option->choices;
@@ -115,13 +117,13 @@ ppdLocalize(ppd_file_t *ppd)		/* I - PPD file */
       {
         if (strcmp(choice->choice, "Custom") ||
 	    !ppdFindCustomOption(ppd, option->keyword))
-	  locattr = ppd_localized_attr(ppd, option->keyword, choice->choice,
-	                               ll_CC, ll);
+	  locattr = _ppdLocalizedAttr(ppd, option->keyword, choice->choice,
+	                              ll_CC);
 	else
 	{
 	  snprintf(ckeyword, sizeof(ckeyword), "Custom%s", option->keyword);
 
-	  locattr = ppd_localized_attr(ppd, ckeyword, "True", ll_CC, ll);
+	  locattr = _ppdLocalizedAttr(ppd, ckeyword, "True", ll_CC);
 	}
 
         if (locattr)
@@ -144,8 +146,8 @@ ppdLocalize(ppd_file_t *ppd)		/* I - PPD file */
     {
       snprintf(ckeyword, sizeof(ckeyword), "ParamCustom%s", coption->keyword);
 
-      if ((locattr = ppd_localized_attr(ppd, ckeyword, cparam->name,
-                                        ll_CC, ll)) != NULL)
+      if ((locattr = _ppdLocalizedAttr(ppd, ckeyword, cparam->name,
+                                       ll_CC)) != NULL)
         strlcpy(cparam->text, locattr->text, sizeof(cparam->text));
     }
   }
@@ -156,8 +158,8 @@ ppdLocalize(ppd_file_t *ppd)		/* I - PPD file */
 
   if ((attr = ppdFindAttr(ppd, "APCustomColorMatchingName", NULL)) != NULL)
   {
-    if ((locattr = ppd_localized_attr(ppd, "APCustomColorMatchingName",
-                                      attr->spec, ll_CC, ll)) != NULL)
+    if ((locattr = _ppdLocalizedAttr(ppd, "APCustomColorMatchingName",
+                                     attr->spec, ll_CC)) != NULL)
       strlcpy(attr->text, locattr->text, sizeof(attr->text));
   }
 
@@ -167,8 +169,8 @@ ppdLocalize(ppd_file_t *ppd)		/* I - PPD file */
   {
     cupsArraySave(ppd->sorted_attrs);
 
-    if ((locattr = ppd_localized_attr(ppd, "cupsICCProfile", attr->spec,
-                                      ll_CC, ll)) != NULL)
+    if ((locattr = _ppdLocalizedAttr(ppd, "cupsICCProfile", attr->spec,
+                                     ll_CC)) != NULL)
       strlcpy(attr->text, locattr->text, sizeof(attr->text));
 
     cupsArrayRestore(ppd->sorted_attrs);
@@ -184,8 +186,8 @@ ppdLocalize(ppd_file_t *ppd)		/* I - PPD file */
   {
     cupsArraySave(ppd->sorted_attrs);
 
-    if ((locattr = ppd_localized_attr(ppd, "APPrinterPreset", attr->spec,
-                                      ll_CC, ll)) != NULL)
+    if ((locattr = _ppdLocalizedAttr(ppd, "APPrinterPreset", attr->spec,
+                                     ll_CC)) != NULL)
       strlcpy(attr->text, locattr->text, sizeof(attr->text));
 
     cupsArrayRestore(ppd->sorted_attrs);
@@ -195,6 +197,45 @@ ppdLocalize(ppd_file_t *ppd)		/* I - PPD file */
 }
 
 
+/*
+ * 'ppdLocalizeAttr()' - Localize an attribute.
+ *
+ * This function uses the current locale to find the localized attribute for
+ * the given main and option keywords.  If no localized version of the
+ * attribute exists for the current locale, the unlocalized version is returned.
+ */
+
+ppd_attr_t *				/* O - Localized attribute or @code NULL@ if none exists */
+ppdLocalizeAttr(ppd_file_t *ppd,	/* I - PPD file */
+		const char *keyword,	/* I - Main keyword */
+		const char *spec)	/* I - Option keyword or @code NULL@ for none */
+{
+  ppd_attr_t	*locattr;		/* Localized attribute */
+  char		ll_CC[6];		/* Language + country locale */
+
+
+ /*
+  * Get the default language...
+  */
+
+  ppd_ll_CC(ll_CC, sizeof(ll_CC));
+
+ /*
+  * Find the localized attribute...
+  */
+
+  if (spec)
+    locattr = _ppdLocalizedAttr(ppd, keyword, spec, ll_CC);
+  else
+    locattr = _ppdLocalizedAttr(ppd, "Translation", keyword, ll_CC);
+
+  if (!locattr)
+    locattr = ppdFindAttr(ppd, keyword, spec);
+
+  return (locattr);
+}
+
+
 /*
  * 'ppdLocalizeIPPReason()' - Get the localized version of a cupsIPPReason
  *                            attribute.
@@ -206,7 +247,7 @@ ppdLocalize(ppd_file_t *ppd)		/* I - PPD file */
  *
  * If no value of the requested scheme can be found, NULL is returned.
  *
- * @since CUPS 1.3@
+ * @since CUPS 1.3/Mac OS X 10.5@
  */
 
 const char *				/* O - Value or NULL if not found */
@@ -217,9 +258,9 @@ ppdLocalizeIPPReason(
     char       *buffer,			/* I - Value buffer */
     size_t     bufsize)			/* I - Size of value buffer */
 {
+  cups_lang_t	*lang;			/* Current language */
   ppd_attr_t	*locattr;		/* Localized attribute */
   char		ll_CC[6],		/* Language + country locale */
-		ll[3],			/* Language locale */
 		*bufptr,		/* Pointer into buffer */
 		*bufend,		/* Pointer to end of buffer */
 		*valptr;		/* Pointer into value */
@@ -242,18 +283,86 @@ ppdLocalizeIPPReason(
   * Get the default language...
   */
 
-  ppd_ll_CC(ll_CC, sizeof(ll_CC), ll, sizeof(ll));
+  lang = ppd_ll_CC(ll_CC, sizeof(ll_CC));
 
  /*
   * Find the localized attribute...
   */
 
-  if ((locattr = ppd_localized_attr(ppd, "cupsIPPReason", reason,
-                                    ll_CC, ll)) == NULL)
+  if ((locattr = _ppdLocalizedAttr(ppd, "cupsIPPReason", reason,
+                                   ll_CC)) == NULL)
     locattr = ppdFindAttr(ppd, "cupsIPPReason", reason);
 
   if (!locattr)
+  {
+    if (lang && (!scheme || !strcmp(scheme, "text")))
+    {
+     /*
+      * Try to localize a standard printer-state-reason keyword...
+      */
+
+      const char *message = NULL;	/* Localized message */
+
+
+      if (!strncmp(reason, "media-needed", 12))
+        message = _("Media tray needs to be filled.");
+      else if (!strncmp(reason, "media-jam", 9))
+        message = _("Media jam!");
+      else if (!strncmp(reason, "offline", 7) ||
+	       !strncmp(reason, "shutdown", 8))
+        message = _("Printer offline.");
+      else if (!strncmp(reason, "toner-low", 9))
+        message = _("Toner low.");
+      else if (!strncmp(reason, "toner-empty", 11))
+        message = _("Out of toner!");
+      else if (!strncmp(reason, "cover-open", 10))
+        message = _("Cover open.");
+      else if (!strncmp(reason, "interlock-open", 14))
+        message = _("Interlock open.");
+      else if (!strncmp(reason, "door-open", 9))
+        message = _("Door open.");
+      else if (!strncmp(reason, "input-tray-missing", 18))
+        message = _("Media tray missing!");
+      else if (!strncmp(reason, "media-low", 9))
+        message = _("Media tray almost empty.");
+      else if (!strncmp(reason, "media-empty", 11))
+        message = _("Media tray empty!");
+      else if (!strncmp(reason, "output-tray-missing", 19))
+        message = _("Output tray missing!");
+      else if (!strncmp(reason, "output-area-almost-full", 23))
+        message = _("Output bin almost full.");
+      else if (!strncmp(reason, "output-area-full", 16))
+        message = _("Output bin full!");
+      else if (!strncmp(reason, "marker-supply-low", 17))
+        message = _("Ink/toner almost empty.");
+      else if (!strncmp(reason, "marker-supply-empty", 19))
+        message = _("Ink/toner empty!");
+      else if (!strncmp(reason, "marker-waste-almost-full", 24))
+        message = _("Ink/toner waste bin almost full.");
+      else if (!strncmp(reason, "marker-waste-full", 17))
+        message = _("Ink/toner waste bin full!");
+      else if (!strncmp(reason, "fuser-over-temp", 15))
+        message = _("Fuser temperature high!");
+      else if (!strncmp(reason, "fuser-under-temp", 16))
+        message = _("Fuser temperature low!");
+      else if (!strncmp(reason, "opc-near-eol", 12))
+        message = _("OPC almost at end-of-life.");
+      else if (!strncmp(reason, "opc-life-over", 13))
+        message = _("OPC at end-of-life!");
+      else if (!strncmp(reason, "developer-low", 13))
+        message = _("Developer almost empty.");
+      else if (!strncmp(reason, "developer-empty", 15))
+        message = _("Developer empty!");
+
+      if (message)
+      {
+        strlcpy(buffer, _cupsLangString(lang, message), bufsize);
+	return (buffer);
+      }
+    }
+
     return (NULL);
+  }
 
  /*
   * Now find the value we need...
@@ -278,9 +387,6 @@ ppdLocalizeIPPReason(
         * Decode text: URI and add to the buffer...
 	*/
 
-        if (bufptr > buffer)
-	  *bufptr++ = ' ';		/* Add leading whitespace */
-
 	valptr += 5;
 
         while (*valptr && !isspace(*valptr & 255) && bufptr < bufend)
@@ -387,15 +493,244 @@ ppdLocalizeIPPReason(
 }
 
 
+/*
+ * 'ppdLocalizeMarkerName()' - Get the localized version of a marker-names
+ *                             attribute value.
+ *
+ * This function uses the current locale to find the corresponding name
+ * text from the attribute value. If no localized text for the requested
+ * name can be found, @code NULL@ is returned.
+ *
+ * @since CUPS 1.4/Mac OS X 10.6@
+ */
+
+const char *				/* O - Value or @code NULL@ if not found */
+ppdLocalizeMarkerName(
+    ppd_file_t *ppd,			/* I - PPD file */
+    const char *name)			/* I - Marker name to look up */
+{
+  ppd_attr_t	*locattr;		/* Localized attribute */
+  char		ll_CC[6];		/* Language + country locale */
+
+
+ /*
+  * Range check input...
+  */
+
+  if (!ppd || !name)
+    return (NULL);
+
+ /*
+  * Get the default language...
+  */
+
+  ppd_ll_CC(ll_CC, sizeof(ll_CC));
+
+ /*
+  * Find the localized attribute...
+  */
+
+  if ((locattr = _ppdLocalizedAttr(ppd, "cupsMarkerName", name,
+                                   ll_CC)) == NULL)
+    locattr = ppdFindAttr(ppd, "cupsMarkerName", name);
+
+  return (locattr ? locattr->text : NULL);
+}
+
+
+/*
+ * '_ppdFreeLanguages()' - Free an array of languages from _ppdGetLanguages.
+ */
+
+void
+_ppdFreeLanguages(
+    cups_array_t *languages)		/* I - Languages array */
+{
+  char	*language;			/* Current language */
+
+
+  for (language = (char *)cupsArrayFirst(languages);
+       language;
+       language = (char *)cupsArrayNext(languages))
+    free(language);
+
+  cupsArrayDelete(languages);
+}
+
+
+/*
+ * '_ppdGetLanguages()' - Get an array of languages from a PPD file.
+ */
+
+cups_array_t *				/* O - Languages array */
+_ppdGetLanguages(ppd_file_t *ppd)	/* I - PPD file */
+{
+  cups_array_t	*languages;		/* Languages array */
+  ppd_attr_t	*attr;			/* cupsLanguages attribute */
+  char		*value,			/* Copy of attribute value */
+		*start,			/* Start of current language */
+		*ptr;			/* Pointer into languages */
+
+
+ /*
+  * See if we have a cupsLanguages attribute...
+  */
+
+  if ((attr = ppdFindAttr(ppd, "cupsLanguages", NULL)) == NULL || !attr->value)
+    return (NULL);
+
+ /*
+  * Yes, load the list...
+  */
+
+  if ((languages = cupsArrayNew((cups_array_func_t)strcmp, NULL)) == NULL)
+    return (NULL);
+
+  if ((value = strdup(attr->value)) == NULL)
+  {
+    cupsArrayDelete(languages);
+    return (NULL);
+  }
+
+  for (ptr = value; *ptr;)
+  {
+   /*
+    * Skip leading whitespace...
+    */
+
+    while (isspace(*ptr & 255))
+      ptr ++;
+
+    if (!*ptr)
+      break;
+
+   /*
+    * Find the end of this language name...
+    */
+
+    for (start = ptr; *ptr && !isspace(*ptr & 255); ptr ++);
+
+    if (*ptr)
+      *ptr++ = '\0';
+
+    if (!strcmp(start, "en"))
+      continue;
+
+    cupsArrayAdd(languages, strdup(start));
+  }
+
+ /*
+  * Free the temporary string and return either an array with one or more
+  * values or a NULL pointer...
+  */
+
+  free(value);
+
+  if (cupsArrayCount(languages) == 0)
+  {
+    cupsArrayDelete(languages);
+    return (NULL);
+  }
+  else
+    return (languages);
+}
+
+
+/*
+ * '_ppdHashName()' - Generate a hash value for a device or profile name.
+ *
+ * This function is primarily used on Mac OS X, but is generally accessible
+ * since cupstestppd needs to check for profile name collisions in PPD files...
+ */
+
+unsigned				/* O - Hash value */
+_ppdHashName(const char *name)		/* I - Name to hash */
+{
+  int		mult;			/* Multiplier */
+  unsigned	hash = 0;		/* Hash value */
+
+
+  for (mult = 1; *name && mult <= 128; mult ++, name ++)
+    hash += (*name & 255) * mult;
+
+  return (hash);
+}
+
+
+/*
+ * '_ppdLocalizedAttr()' - Find a localized attribute.
+ */
+
+ppd_attr_t *				/* O - Localized attribute or NULL */
+_ppdLocalizedAttr(ppd_file_t *ppd,	/* I - PPD file */
+		  const char *keyword,	/* I - Main keyword */
+		  const char *spec,	/* I - Option keyword */
+		  const char *ll_CC)	/* I - Language + country locale */
+{
+  char		lkeyword[PPD_MAX_NAME];	/* Localization keyword */
+  ppd_attr_t	*attr;			/* Current attribute */
+
+
+  DEBUG_printf(("4_ppdLocalizedAttr(ppd=%p, keyword=\"%s\", spec=\"%s\", "
+                "ll_CC=\"%s\")", ppd, keyword, spec, ll_CC));
+
+ /*
+  * Look for Keyword.ll_CC, then Keyword.ll...
+  */
+
+  snprintf(lkeyword, sizeof(lkeyword), "%s.%s", ll_CC, keyword);
+  if ((attr = ppdFindAttr(ppd, lkeyword, spec)) == NULL)
+  {
+    snprintf(lkeyword, sizeof(lkeyword), "%2.2s.%s", ll_CC, keyword);
+    attr = ppdFindAttr(ppd, lkeyword, spec);
+
+    if (!attr)
+    {
+      if (!strncmp(ll_CC, "ja", 2))
+      {
+       /*
+	* Due to a bug in the CUPS DDK 1.1.0 ppdmerge program, Japanese
+	* PPD files were incorrectly assigned "jp" as the locale name
+	* instead of "ja".  Support both the old (incorrect) and new
+	* locale names for Japanese...
+	*/
+
+	snprintf(lkeyword, sizeof(lkeyword), "jp.%s", keyword);
+	attr = ppdFindAttr(ppd, lkeyword, spec);
+      }
+      else if (!strncmp(ll_CC, "no", 2))
+      {
+       /*
+	* Norway has two languages, "Bokmal" (the primary one)
+	* and "Nynorsk" (new Norwegian); we map "no" to "nb" here as
+	* recommended by the locale folks...
+	*/
+
+	snprintf(lkeyword, sizeof(lkeyword), "nb.%s", keyword);
+	attr = ppdFindAttr(ppd, lkeyword, spec);
+      }
+    }
+  }
+
+#ifdef DEBUG
+  if (attr)
+    DEBUG_printf(("5_ppdLocalizedAttr: *%s %s/%s: \"%s\"\n", attr->name,
+                  attr->spec, attr->text, attr->value ? attr->value : ""));
+  else
+    DEBUG_puts("5_ppdLocalizedAttr: NOT FOUND");
+#endif /* DEBUG */
+
+  return (attr);
+}
+
+
 /*
  * 'ppd_ll_CC()' - Get the current locale names.
  */
 
-static void
+static cups_lang_t *			/* O - Current language */
 ppd_ll_CC(char *ll_CC,			/* O - Country-specific locale name */
-          int  ll_CC_size,		/* I - Size of country-specific name */
-          char *ll,			/* O - Generic locale name */
-          int  ll_size)			/* I - Size of generic name */
+          int  ll_CC_size)		/* I - Size of country-specific name */
 {
   cups_lang_t	*lang;			/* Current language */
 
@@ -407,8 +742,7 @@ ppd_ll_CC(char *ll_CC,			/* O - Country-specific locale name */
   if ((lang = cupsLangDefault()) == NULL)
   {
     strlcpy(ll_CC, "en_US", ll_CC_size);
-    strlcpy(ll, "en", ll_size);
-    return;
+    return (NULL);
   }
 
  /*
@@ -416,9 +750,6 @@ ppd_ll_CC(char *ll_CC,			/* O - Country-specific locale name */
   */
 
   strlcpy(ll_CC, lang->language, ll_CC_size);
-  strlcpy(ll, lang->language, ll_size);
-
-  DEBUG_printf(("ll_CC=\"%s\", ll=\"%s\"\n", ll_CC, ll));
 
   if (strlen(ll_CC) == 2)
   {
@@ -439,80 +770,12 @@ ppd_ll_CC(char *ll_CC,			/* O - Country-specific locale name */
       strlcpy(ll_CC, "zh_CN", ll_CC_size);
   }
 
-  DEBUG_printf(("ppd_ll_CC: lang->language=\"%s\", ll=\"%s\", ll_CC=\"%s\"...\n",
-                lang->language, ll, ll_CC));
+  DEBUG_printf(("8ppd_ll_CC: lang->language=\"%s\", ll_CC=\"%s\"...",
+                lang->language, ll_CC));
+  return (lang);
 }
 
 
 /*
- * 'ppd_localized_attr()' - Find a localized attribute.
- */
-
-static ppd_attr_t *			/* O - Localized attribute or NULL */
-ppd_localized_attr(ppd_file_t *ppd,	/* I - PPD file */
-		   const char *keyword,	/* I - Main keyword */
-		   const char *spec,	/* I - Option keyword */
-		   const char *ll_CC,	/* I - Language + country locale */
-		   const char *ll)	/* I - Language locale */
-{
-  char		lkeyword[PPD_MAX_NAME];	/* Localization keyword */
-  ppd_attr_t	*attr;			/* Current attribute */
-
-
-  DEBUG_printf(("ppd_text(ppd=%p, keyword=\"%s\", spec=\"%s\", "
-                "ll_CC=\"%s\", ll=\"%s\")\n",
-                ppd, keyword, spec, ll_CC, ll));
-
- /*
-  * Look for Keyword.ll_CC, then Keyword.ll...
-  */
-
-  snprintf(lkeyword, sizeof(lkeyword), "%s.%s", ll_CC, keyword);
-  if ((attr = ppdFindAttr(ppd, lkeyword, spec)) == NULL)
-  {
-    snprintf(lkeyword, sizeof(lkeyword), "%s.%s", ll, keyword);
-    attr = ppdFindAttr(ppd, lkeyword, spec);
-
-    if (!attr)
-    {
-      if (!strcmp(ll, "ja"))
-      {
-       /*
-	* Due to a bug in the CUPS DDK 1.1.0 ppdmerge program, Japanese
-	* PPD files were incorrectly assigned "jp" as the locale name
-	* instead of "ja".  Support both the old (incorrect) and new
-	* locale names for Japanese...
-	*/
-
-	snprintf(lkeyword, sizeof(lkeyword), "jp.%s", keyword);
-	attr = ppdFindAttr(ppd, lkeyword, spec);
-      }
-      else if (!strcmp(ll, "no"))
-      {
-       /*
-	* Norway has two languages, "Bokmal" (the primary one)
-	* and "Nynorsk" (new Norwegian); we map "no" to "nb" here as
-	* recommended by the locale folks...
-	*/
-
-	snprintf(lkeyword, sizeof(lkeyword), "nb.%s", keyword);
-	attr = ppdFindAttr(ppd, lkeyword, spec);
-      }
-    }
-  }
-
-#ifdef DEBUG
-  if (attr)
-    printf("    *%s %s/%s: \"%s\"\n", attr->name, attr->spec, attr->text,
-           attr->value ? attr->value : "");
-  else
-    puts("    NOT FOUND");
-#endif /* DEBUG */
-
-  return (attr);
-}
-
-
-/*
- * End of "$Id: localize.c 8374 2009-02-20 16:42:13Z mike $".
+ * End of "$Id: localize.c 8627 2009-05-13 21:39:17Z mike $".
  */
