@@ -1,25 +1,16 @@
 /*
- * "$Id: hpgl-prolog.c 6003 2006-10-02 16:26:04Z mike $"
+ * "$Id: hpgl-prolog.c 6649 2007-07-11 21:46:42Z mike $"
  *
  *   HP-GL/2 prolog routines for for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1993-2005 by Easy Software Products.
+ *   Copyright 2007 by Apple Inc.
+ *   Copyright 1993-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  *
@@ -88,7 +79,8 @@ OutputProlog(char  *title,	/* I - Job title */
 
   if ((prolog = fopen(filename, "r")) == NULL)
   {
-    fprintf(stderr, "ERROR: Unable to open HPGL prolog \"%s\" for reading - %s\n",
+    fprintf(stderr,
+            "DEBUG: Unable to open HPGL prolog \"%s\" for reading - %s\n",
             filename, strerror(errno));
     exit(1);
   }
@@ -373,5 +365,5 @@ Outputf(const char *format,	/* I - Printf-style string */
 
 
 /*
- * End of "$Id: hpgl-prolog.c 6003 2006-10-02 16:26:04Z mike $".
+ * End of "$Id: hpgl-prolog.c 6649 2007-07-11 21:46:42Z mike $".
  */
