@@ -1,25 +1,16 @@
 /*
- * "$Id: classes.h 4719 2005-09-28 21:12:44Z mike $"
+ * "$Id: classes.h 7964 2008-09-19 20:03:36Z mike $"
  *
  *   Printer class definitions for the Common UNIX Printing System (CUPS).
  *
+ *   Copyright 2007-2008 by Apple Inc.
  *   Copyright 1997-2005 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  */
 
 
@@ -33,7 +24,6 @@ extern void		cupsdAddPrinterToClass(cupsd_printer_t *c,
 extern void		cupsdDeletePrinterFromClass(cupsd_printer_t *c,
 			                            cupsd_printer_t *p);
 extern void		cupsdDeletePrinterFromClasses(cupsd_printer_t *p);
-extern void		cupsdDeleteAllClasses(void);
 extern cupsd_printer_t	*cupsdFindAvailablePrinter(const char *name);
 extern cupsd_printer_t	*cupsdFindClass(const char *name);
 extern void		cupsdLoadAllClasses(void);
@@ -42,5 +32,5 @@ extern void		cupsdUpdateImplicitClasses(void);
 
 
 /*
- * End of "$Id: classes.h 4719 2005-09-28 21:12:44Z mike $".
+ * End of "$Id: classes.h 7964 2008-09-19 20:03:36Z mike $".
  */
