@@ -1,5 +1,5 @@
 /*
- * "$Id: cups-driverd.cxx 8809 2009-08-31 21:26:25Z mike $"
+ * "$Id: cups-driverd.cxx 8955 2010-01-18 16:18:29Z mike $"
  *
  *   PPD/driver support for the Common UNIX Printing System (CUPS).
  *
@@ -1991,7 +1991,8 @@ load_drv(const char  *filename,		/* I - Actual filename */
 	  type = PPD_TYPE_PDF;
     }
 
-    for (product = (ppdcAttr *)d->attrs->first(), products_found = 0;
+    for (product = (ppdcAttr *)d->attrs->first(), products_found = 0,
+             ppd = NULL;
          product;
 	 product = (ppdcAttr *)d->attrs->next())
       if (!strcmp(product->name->value, "Product"))
@@ -2476,5 +2477,5 @@ regex_string(const char *s)		/* I - String to compare */
 
 
 /*
- * End of "$Id: cups-driverd.cxx 8809 2009-08-31 21:26:25Z mike $".
+ * End of "$Id: cups-driverd.cxx 8955 2010-01-18 16:18:29Z mike $".
  */
