@@ -1,25 +1,16 @@
 /*
- * "$Id: cupsd.h 5305 2006-03-18 03:05:12Z mike $"
+ * "$Id: cupsd.h 8481 2009-04-02 22:53:56Z mike $"
  *
  *   Main header file for the Common UNIX Printing System (CUPS) scheduler.
  *
- *   Copyright 1997-2006 by Easy Software Products, all rights reserved.
+ *   Copyright 2007-2009 by Apple Inc.
+ *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
  *   "LICENSE" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  */
 
 
@@ -101,7 +92,7 @@ extern const char *cups_hstrerror(int);
 #define DEFAULT_HISTORY		1	/* Preserve job history? */
 #define DEFAULT_FILES		0	/* Preserve job files? */
 #define DEFAULT_TIMEOUT		300	/* Timeout during requests/updates */
-#define DEFAULT_KEEPALIVE	60	/* Timeout between requests */
+#define DEFAULT_KEEPALIVE	30	/* Timeout between requests */
 #define DEFAULT_INTERVAL	30	/* Interval between browse updates */
 #define DEFAULT_CHARSET		"utf-8"	/* Default charset */
 
@@ -150,29 +141,59 @@ extern const char *cups_hstrerror(int);
 #define RELOAD_CUPSD	2		/* Reload only cupsd.conf */
 
 
+/*
+ * Select callback function type...
+ */
+
+typedef void (*cupsd_selfunc_t)(void *data);
+
+
 /*
  * Globals...
  */
 
-VAR int			MaxFDs,		/* Maximum number of files */
-			SetSize;	/* The size of the input/output sets */
-VAR fd_set		*InputSet,	/* Input files for select() */
-			*OutputSet;	/* Output files for select() */
+VAR int			TestConfigFile	VALUE(0),
+					/* Test the cupsd.conf file? */
+			UseProfiles	VALUE(1);
+					/* Use security profiles for child procs? */
+VAR int			MaxFDs		VALUE(0);
+					/* Maximum number of files */
 
 VAR time_t		ReloadTime	VALUE(0);
 					/* Time of reload request... */
 VAR int			NeedReload	VALUE(RELOAD_ALL);
 					/* Need to load configuration? */
+VAR void		*DefaultProfile	VALUE(0);
+					/* Default security profile */
+
+#ifdef HAVE_GSSAPI
+VAR int			KerberosInitialized	VALUE(0);
+					/* Has Kerberos been initialized? */
+VAR krb5_context	KerberosContext VALUE(NULL);
+					/* Kerberos context for credentials */
+#endif /* HAVE_GSSAPI */
+
 #ifdef HAVE_LAUNCH_H
 VAR int			Launchd		VALUE(0);
 					/* Running from launchd */
 #endif /* HAVE_LAUNCH_H */
 
+#if defined(__APPLE__) && defined(HAVE_DLFCN_H)
+typedef int (*PSQUpdateQuotaProcPtr)(const char *printer, const char *info, 
+                                     const char *user, int nPages, int options);
+VAR PSQUpdateQuotaProcPtr PSQUpdateQuotaProc
+					VALUE(0);
+					/* Apple PrintService quota function */
+#endif /* __APPLE__ && HAVE_DLFCN_H */
+
+
+
 
 /*
  * Prototypes...
  */
 
+extern void	cupsdCheckProcess(void);
 extern void	cupsdClearString(char **s);
 extern void	cupsdHoldSignals(void);
 extern void	cupsdReleaseSignals(void);
@@ -196,13 +217,30 @@ __attribute__ ((__format__ (__printf__, 2, 3)))
 #endif /* __GNUC__ */
 ;
 
+extern void	*cupsdCreateProfile(int job_id);
+extern void	cupsdDestroyProfile(void *profile);
 extern int	cupsdEndProcess(int pid, int force);
-extern const char *cupsdFinishProcess(int pid, char *name, int namelen);
+extern const char *cupsdFinishProcess(int pid, char *name, int namelen,
+		                      int *job_id);
 extern int	cupsdStartProcess(const char *command, char *argv[],
 				  char *envp[], int infd, int outfd,
-				  int errfd, int backfd, int root, int *pid);
+				  int errfd, int backfd, int sidefd,
+				  int root, void *profile, cupsd_job_t *job,
+				  int *pid);
+
+extern int	cupsdAddSelect(int fd, cupsd_selfunc_t read_cb,
+		               cupsd_selfunc_t write_cb, void *data);
+extern int	cupsdDoSelect(long timeout);
+#ifdef CUPSD_IS_SELECTING
+extern int	cupsdIsSelecting(int fd);
+#endif /* CUPSD_IS_SELECTING */
+extern void	cupsdRemoveSelect(int fd);
+extern void	cupsdStartSelect(void);
+extern void	cupsdStopSelect(void);
+
+extern int	cupsdRemoveFile(const char *filename);
 
 
 /*
- * End of "$Id: cupsd.h 5305 2006-03-18 03:05:12Z mike $".
+ * End of "$Id: cupsd.h 8481 2009-04-02 22:53:56Z mike $".
  */
